@@ -1,26 +1,26 @@
 from functools import reduce
+from itertools import product
 
 from sympy.core.basic import Basic
 from sympy.core.containers import Tuple
 from sympy.core.expr import Expr
 from sympy.core.function import Lambda
 from sympy.core.logic import fuzzy_not, fuzzy_or, fuzzy_and
-from sympy.core.numbers import oo, Integer
-from sympy.core.relational import Eq
+from sympy.core.mod import Mod
+from sympy.core.numbers import oo, igcd, Rational
+from sympy.core.relational import Eq, is_eq
 from sympy.core.singleton import Singleton, S
 from sympy.core.symbol import Dummy, symbols, Symbol
-from sympy.core.sympify import _sympify, sympify, converter
-from sympy.logic.boolalg import And
-from sympy.sets.sets import (Set, Interval, Union, FiniteSet,
-    ProductSet)
+from sympy.core.sympify import _sympify, sympify, _sympy_converter
+from sympy.logic.boolalg import And, Or
+from .sets import Set, Interval, Union, FiniteSet, ProductSet
 from sympy.utilities.misc import filldedent
-from sympy.utilities.iterables import cartes
 
 
 class Rationals(Set, metaclass=Singleton):
     """
     Represents the rational numbers. This set is also available as
-    the Singleton, S.Rationals.
+    the singleton ``S.Rationals``.
 
     Examples
     ========
@@ -45,7 +45,6 @@ class Rationals(Set, metaclass=Singleton):
         return other.is_rational
 
     def __iter__(self):
-        from sympy.core.numbers import igcd, Rational
         yield S.Zero
         yield S.One
         yield S.NegativeOne
@@ -68,7 +67,7 @@ class Naturals(Set, metaclass=Singleton):
     """
     Represents the natural numbers (or counting numbers) which are all
     positive integers starting from 1. This set is also available as
-    the Singleton, S.Naturals.
+    the singleton ``S.Naturals``.
 
     Examples
     ========
@@ -158,7 +157,7 @@ class Naturals0(Naturals):
 class Integers(Set, metaclass=Singleton):
     """
     Represents all integers: positive, negative and zero. This set is also
-    available as the Singleton, S.Integers.
+    available as the singleton ``S.Integers``.
 
     Examples
     ========
@@ -231,7 +230,7 @@ class Reals(Interval, metaclass=Singleton):
     Represents all real numbers
     from negative infinity to positive infinity,
     including all integer, rational and irrational numbers.
-    This set is also available as the Singleton, S.Reals.
+    This set is also available as the singleton ``S.Reals``.
 
 
     Examples
@@ -255,8 +254,21 @@ class Reals(Interval, metaclass=Singleton):
 
     ComplexRegion
     """
-    def __new__(cls):
-        return Interval.__new__(cls, S.NegativeInfinity, S.Infinity)
+    @property
+    def start(self):
+        return S.NegativeInfinity
+
+    @property
+    def end(self):
+        return S.Infinity
+
+    @property
+    def left_open(self):
+        return True
+
+    @property
+    def right_open(self):
+        return True
 
     def __eq__(self, other):
         return other == Interval(S.NegativeInfinity, S.Infinity)
@@ -274,15 +286,14 @@ class ImageSet(Set):
     a complex region.
 
     This function is not normally called directly, but is called
-    from `imageset`.
+    from ``imageset``.
 
 
     Examples
     ========
 
     >>> from sympy import Symbol, S, pi, Dummy, Lambda
-    >>> from sympy.sets.sets import FiniteSet, Interval
-    >>> from sympy.sets.fancysets import ImageSet
+    >>> from sympy import FiniteSet, ImageSet, Interval
 
     >>> x = Symbol('x')
     >>> N = S.Naturals
@@ -293,7 +304,7 @@ class ImageSet(Set):
     False
 
     >>> FiniteSet(0, 1, 2, 3, 4, 5, 6, 7, 9, 10).intersect(squares)
-    FiniteSet(1, 4, 9)
+    {1, 4, 9}
 
     >>> square_iterable = iter(squares)
     >>> for i in range(4):
@@ -304,7 +315,7 @@ class ImageSet(Set):
     16
 
     If you want to get value for `x` = 2, 1/2 etc. (Please check whether the
-    `x` value is in `base_set` or not before passing it as args)
+    `x` value is in ``base_set`` or not before passing it as args)
 
     >>> squares.lamda(2)
     4
@@ -315,7 +326,7 @@ class ImageSet(Set):
     >>> solutions = ImageSet(Lambda(n, n*pi), S.Integers) # solutions of sin(x) = 0
     >>> dom = Interval(-1, 1)
     >>> dom.intersect(solutions)
-    FiniteSet(0)
+    {0}
 
     See Also
     ========
@@ -487,18 +498,18 @@ class ImageSet(Set):
             base_set = self.base_sets[0]
             return SetExpr(base_set)._eval_func(f).set
         if all(s.is_FiniteSet for s in self.base_sets):
-            return FiniteSet(*(f(*a) for a in cartes(*self.base_sets)))
+            return FiniteSet(*(f(*a) for a in product(*self.base_sets)))
         return self
 
 
 class Range(Set):
     """
-    Represents a range of integers. Can be called as Range(stop),
-    Range(start, stop), or Range(start, stop, step); when stop is
+    Represents a range of integers. Can be called as ``Range(stop)``,
+    ``Range(start, stop)``, or ``Range(start, stop, step)``; when ``step`` is
     not given it defaults to 1.
 
-    `Range(stop)` is the same as `Range(0, stop, 1)` and the stop value
-    (juse as for Python ranges) is not included in the Range values.
+    ``Range(stop)`` is the same as ``Range(0, stop, 1)`` and the stop value
+    (just as for Python ranges) is not included in the Range values.
 
         >>> from sympy import Range
         >>> list(Range(3))
@@ -531,9 +542,9 @@ class Range(Set):
         >>> next(iter(r.reversed))
         0
 
-    Although Range is a set (and supports the normal set
+    Although ``Range`` is a :class:`Set` (and supports the normal set
     operations) it maintains the order of the elements and can
-    be used in contexts where `range` would be used.
+    be used in contexts where ``range`` would be used.
 
         >>> from sympy import Interval
         >>> Range(0, 10, 2).intersect(Interval(3, 7))
@@ -571,11 +582,9 @@ class Range(Set):
         >>> r.inf
         n
         >>> pprint(r)
-        {n, n + 3, ..., n + 17}
+        {n, n + 3, ..., n + 18}
     """
 
-    is_iterable = True
-
     def __new__(cls, *args):
         from sympy.functions.elementary.integers import ceiling
         if len(args) == 1:
@@ -598,6 +607,8 @@ class Range(Set):
                         w.has(Symbol) and w.is_integer != False):
                     ok.append(w)
                 elif not w.is_Integer:
+                    if w.is_infinite:
+                        raise ValueError('infinite symbols not allowed')
                     raise ValueError
                 else:
                     ok.append(w)
@@ -610,10 +621,25 @@ class Range(Set):
 
         null = False
         if any(i.has(Symbol) for i in (start, stop, step)):
-            if start == stop:
+            dif = stop - start
+            n = dif/step
+            if n.is_Rational:
+                from sympy.functions.elementary.integers import floor
+                if dif == 0:
+                    null = True
+                else:  # (x, x + 5, 2) or (x, 3*x, x)
+                    n = floor(n)
+                    end = start + n*step
+                    if dif.is_Rational:  # (x, x + 5, 2)
+                        if (end - stop).is_negative:
+                            end += step
+                    else:  # (x, 3*x, x)
+                        if (end/stop - 1).is_negative:
+                            end += step
+            elif n.is_extended_negative:
                 null = True
             else:
-                end = stop
+                end = stop  # other methods like sup and reversed must fail
         elif start.is_infinite:
             span = step*(stop - start)
             if span is S.NaN or span <= 0:
@@ -631,8 +657,8 @@ class Range(Set):
             if n <= 0:
                 null = True
             elif oostep:
-                end = start + 1
-                step = S.One  # make it a canonical single step
+                step = S.One  # make it canonical
+                end = start + step
             else:
                 end = start + n*step
         if null:
@@ -656,34 +682,42 @@ class Range(Set):
         Range(9, -1, -1)
         """
         if self.has(Symbol):
-            _ = self.size  # validate
-        if not self:
+            n = (self.stop - self.start)/self.step
+            if not n.is_extended_positive or not all(
+                    i.is_integer or i.is_infinite for i in self.args):
+                raise ValueError('invalid method for symbolic range')
+        if self.start == self.stop:
             return self
         return self.func(
             self.stop - self.step, self.start - self.step, -self.step)
 
     def _contains(self, other):
-        if not self:
+        if self.start == self.stop:
             return S.false
         if other.is_infinite:
             return S.false
         if not other.is_integer:
             return other.is_integer
         if self.has(Symbol):
-            try:
-                _ = self.size  # validate
-            except ValueError:
+            n = (self.stop - self.start)/self.step
+            if not n.is_extended_positive or not all(
+                    i.is_integer or i.is_infinite for i in self.args):
                 return
+        else:
+            n = self.size
         if self.start.is_finite:
             ref = self.start
         elif self.stop.is_finite:
             ref = self.stop
         else:  # both infinite; step is +/- 1 (enforced by __new__)
             return S.true
-        if self.size == 1:
+        if n == 1:
             return Eq(other, self[0])
         res = (ref - other) % self.step
         if res == S.Zero:
+            if self.has(Symbol):
+                d = Dummy('i')
+                return self.as_relational(d).subs(d, other)
             return And(other >= self.inf, other <= self.sup)
         elif res.is_Integer:  # off sequence
             return S.false
@@ -691,20 +725,34 @@ class Range(Set):
             return None
 
     def __iter__(self):
-        if self.has(Symbol):
-            _ = self.size  # validate
+        n = self.size  # validate
+        if not (n.has(S.Infinity) or n.has(S.NegativeInfinity) or n.is_Integer):
+            raise TypeError("Cannot iterate over symbolic Range")
         if self.start in [S.NegativeInfinity, S.Infinity]:
             raise TypeError("Cannot iterate over Range with infinite start")
-        elif self:
+        elif self.start != self.stop:
             i = self.start
-            step = self.step
+            if n.is_infinite:
+                while True:
+                    yield i
+                    i += self.step
+            else:
+                for _ in range(n):
+                    yield i
+                    i += self.step
 
-            while True:
-                if (step > 0 and not (self.start <= i < self.stop)) or \
-                   (step < 0 and not (self.stop < i <= self.start)):
-                    break
-                yield i
-                i += step
+    @property
+    def is_iterable(self):
+        # Check that size can be determined, used by __iter__
+        dif = self.stop - self.start
+        n = dif/self.step
+        if not (n.has(S.Infinity) or n.has(S.NegativeInfinity) or n.is_Integer):
+            return False
+        if self.start in [S.NegativeInfinity, S.Infinity]:
+            return False
+        if not (n.is_extended_nonnegative and all(i.is_integer for i in self.args)):
+            return False
+        return True
 
     def __len__(self):
         rv = self.size
@@ -714,16 +762,16 @@ class Range(Set):
 
     @property
     def size(self):
-        if not self:
+        if self.start == self.stop:
             return S.Zero
         dif = self.stop - self.start
-        if self.has(Symbol):
-            if dif.has(Symbol) or self.step.has(Symbol) or (
-                    not self.start.is_integer and not self.stop.is_integer):
-                raise ValueError('invalid method for symbolic range')
-        if dif.is_infinite:
+        n = dif/self.step
+        if n.is_infinite:
             return S.Infinity
-        return Integer(abs(dif//self.step))
+        if  n.is_extended_nonnegative and all(i.is_integer for i in self.args):
+            from sympy.functions.elementary.integers import floor
+            return abs(floor(n))
+        raise ValueError('Invalid method for symbolic Range')
 
     @property
     def is_finite_set(self):
@@ -731,8 +779,21 @@ class Range(Set):
             return True
         return self.size.is_finite
 
+    @property
+    def is_empty(self):
+        try:
+            return self.size.is_zero
+        except ValueError:
+            return None
+
     def __bool__(self):
-        return self.start != self.stop
+        # this only distinguishes between definite null range
+        # and non-null/unknown null; getting True doesn't mean
+        # that it actually is not null
+        b = is_eq(self.start, self.stop)
+        if b is None:
+            raise ValueError('cannot tell if Range is null or not')
+        return not bool(b)
 
     def __getitem__(self, i):
         from sympy.functions.elementary.integers import ceiling
@@ -746,6 +807,8 @@ class Range(Set):
             "with an infinite value"
         if isinstance(i, slice):
             if self.size.is_finite:  # validates, too
+                if self.start == self.stop:
+                    return Range(0)
                 start, stop, step = i.indices(self.size)
                 n = ceiling((stop - start)/step)
                 if n <= 0:
@@ -846,44 +909,45 @@ class Range(Set):
                 elif start > 0:
                     raise ValueError(ooslice)
         else:
-            if not self:
+            if self.start == self.stop:
                 raise IndexError('Range index out of range')
+            if not (all(i.is_integer or i.is_infinite
+                    for i in self.args) and ((self.stop - self.start)/
+                    self.step).is_extended_positive):
+                raise ValueError('Invalid method for symbolic Range')
             if i == 0:
                 if self.start.is_infinite:
                     raise ValueError(ooslice)
-                if self.has(Symbol):
-                    if (self.stop > self.start) == self.step.is_positive and self.step.is_positive is not None:
-                        pass
-                    else:
-                        _ = self.size  # validate
                 return self.start
             if i == -1:
                 if self.stop.is_infinite:
                     raise ValueError(ooslice)
-                n = self.stop - self.step
-                if n.is_Integer or (
-                        n.is_integer and (
-                            (n - self.start).is_nonnegative ==
-                            self.step.is_positive)):
-                    return n
-            _ = self.size  # validate
+                return self.stop - self.step
+            n = self.size  # must be known for any other index
             rv = (self.stop if i < 0 else self.start) + i*self.step
             if rv.is_infinite:
                 raise ValueError(ooslice)
-            if rv < self.inf or rv > self.sup:
-                raise IndexError("Range index out of range")
-            return rv
+            val = (rv - self.start)/self.step
+            rel = fuzzy_or([val.is_infinite,
+                            fuzzy_and([val.is_nonnegative, (n-val).is_nonnegative])])
+            if rel:
+                return rv
+            if rel is None:
+                raise ValueError('Invalid method for symbolic Range')
+            raise IndexError("Range index out of range")
 
     @property
     def _inf(self):
         if not self:
-            raise NotImplementedError
+            return S.EmptySet.inf
         if self.has(Symbol):
-            if self.step.is_positive:
-                return self[0]
-            elif self.step.is_negative:
-                return self[-1]
-            _ = self.size  # validate
+            if all(i.is_integer or i.is_infinite for i in self.args):
+                dif = self.stop - self.start
+                if self.step.is_positive and dif.is_positive:
+                    return self.start
+                elif self.step.is_negative and dif.is_negative:
+                    return self.stop - self.step
+            raise ValueError('invalid method for symbolic range')
         if self.step > 0:
             return self.start
         else:
@@ -892,13 +956,15 @@ class Range(Set):
     @property
     def _sup(self):
         if not self:
-            raise NotImplementedError
+            return S.EmptySet.sup
         if self.has(Symbol):
-            if self.step.is_positive:
-                return self[-1]
-            elif self.step.is_negative:
-                return self[0]
-            _ = self.size  # validate
+            if all(i.is_integer or i.is_infinite for i in self.args):
+                dif = self.stop - self.start
+                if self.step.is_positive and dif.is_positive:
+                    return self.stop - self.step
+                elif self.step.is_negative and dif.is_negative:
+                    return self.start
+            raise ValueError('invalid method for symbolic range')
         if self.step > 0:
             return self.stop - self.step
         else:
@@ -910,24 +976,46 @@ class Range(Set):
 
     def as_relational(self, x):
         """Rewrite a Range in terms of equalities and logic operators. """
-        from sympy.functions.elementary.integers import floor
-        if self.size == 1:
-            return Eq(x, self[0])
+        if self.start.is_infinite:
+            assert not self.stop.is_infinite  # by instantiation
+            a = self.reversed.start
         else:
-            return And(
-                Eq(x, floor(x)),
-                x >= self.inf if self.inf in self else x > self.inf,
-                x <= self.sup if self.sup in self else x < self.sup)
+            a = self.start
+        step = self.step
+        in_seq = Eq(Mod(x - a, step), 0)
+        ints = And(Eq(Mod(a, 1), 0), Eq(Mod(step, 1), 0))
+        n = (self.stop - self.start)/self.step
+        if n == 0:
+            return S.EmptySet.as_relational(x)
+        if n == 1:
+            return And(Eq(x, a), ints)
+        try:
+            a, b = self.inf, self.sup
+        except ValueError:
+            a = None
+        if a is not None:
+            range_cond = And(
+                x > a if a.is_infinite else x >= a,
+                x < b if b.is_infinite else x <= b)
+        else:
+            a, b = self.start, self.stop - self.step
+            range_cond = Or(
+                And(self.step >= 1, x > a if a.is_infinite else x >= a,
+                x < b if b.is_infinite else x <= b),
+                And(self.step <= -1, x < a if a.is_infinite else x <= a,
+                x > b if b.is_infinite else x >= b))
+        return And(in_seq, ints, range_cond)
 
-converter[range] = lambda r: Range(r.start, r.stop, r.step)
+
+_sympy_converter[range] = lambda r: Range(r.start, r.stop, r.step)
 
 def normalize_theta_set(theta):
-    """
-    Normalize a Real Set `theta` in the Interval [0, 2*pi). It returns
+    r"""
+    Normalize a Real Set `theta` in the interval `[0, 2\pi)`. It returns
     a normalized value of theta in the Set. For Interval, a maximum of
-    one cycle [0, 2*pi], is returned i.e. for theta equal to [0, 10*pi],
-    returned normalized value would be [0, 2*pi). As of now intervals
-    with end points as non-multiples of `pi` is not supported.
+    one cycle $[0, 2\pi]$, is returned i.e. for theta equal to $[0, 10\pi]$,
+    returned normalized value would be $[0, 2\pi)$. As of now intervals
+    with end points as non-multiples of ``pi`` is not supported.
 
     Raises
     ======
@@ -956,7 +1044,7 @@ def normalize_theta_set(theta):
     >>> normalize_theta_set(Interval(-3*pi/2, -pi/2))
     Interval(pi/2, 3*pi/2)
     >>> normalize_theta_set(FiniteSet(0, pi, 3*pi))
-    FiniteSet(0, pi)
+    {0, pi}
 
     """
     from sympy.functions.elementary.trigonometric import _pi_coeff as coeff
@@ -1007,33 +1095,30 @@ def normalize_theta_set(theta):
 
 
 class ComplexRegion(Set):
-    """
+    r"""
     Represents the Set of all Complex Numbers. It can represent a
     region of Complex Plane in both the standard forms Polar and
     Rectangular coordinates.
 
     * Polar Form
       Input is in the form of the ProductSet or Union of ProductSets
-      of the intervals of r and theta, & use the flag polar=True.
+      of the intervals of ``r`` and ``theta``, and use the flag ``polar=True``.
 
-    Z = {z in C | z = r*[cos(theta) + I*sin(theta)], r in [r], theta in [theta]}
+      .. math:: Z = \{z \in \mathbb{C} \mid z = r\times (\cos(\theta) + I\sin(\theta)), r \in [\texttt{r}], \theta \in [\texttt{theta}]\}
 
     * Rectangular Form
       Input is in the form of the ProductSet or Union of ProductSets
-      of interval of x and y the of the Complex numbers in a Plane.
+      of interval of x and y, the real and imaginary parts of the Complex numbers in a plane.
       Default input type is in rectangular form.
 
-    Z = {z in C | z = x + I*y, x in [Re(z)], y in [Im(z)]}
+    .. math:: Z = \{z \in \mathbb{C} \mid z = x + Iy, x \in [\operatorname{re}(z)], y \in [\operatorname{im}(z)]\}
 
     Examples
     ========
 
-    >>> from sympy.sets.fancysets import ComplexRegion
-    >>> from sympy.sets import Interval
-    >>> from sympy import S, I, Union
+    >>> from sympy import ComplexRegion, Interval, S, I, Union
     >>> a = Interval(2, 3)
     >>> b = Interval(4, 6)
-    >>> c = Interval(1, 8)
     >>> c1 = ComplexRegion(a*b)  # Rectangular Form
     >>> c1
     CartesianComplexRegion(ProductSet(Interval(2, 3), Interval(4, 6)))
@@ -1042,6 +1127,7 @@ class ComplexRegion(Set):
       surrounded by the coordinates (2, 4), (3, 4), (3, 6) and
       (2, 6), of the four vertices.
 
+    >>> c = Interval(1, 8)
     >>> c2 = ComplexRegion(Union(a*b, b*c))
     >>> c2
     CartesianComplexRegion(Union(ProductSet(Interval(2, 3), Interval(4, 6)), ProductSet(Interval(4, 6), Interval(1, 8))))
@@ -1235,7 +1321,7 @@ class ComplexRegion(Set):
         >>> from sympy import Interval, ComplexRegion
         >>> unit = Interval(0,1)
         >>> ComplexRegion.from_real(unit)
-        CartesianComplexRegion(ProductSet(Interval(0, 1), FiniteSet(0)))
+        CartesianComplexRegion(ProductSet(Interval(0, 1), {0}))
 
         """
         if not sets.is_subset(S.Reals):
@@ -1245,14 +1331,13 @@ class ComplexRegion(Set):
 
     def _contains(self, other):
         from sympy.functions import arg, Abs
-        from sympy.core.containers import Tuple
         other = sympify(other)
         isTuple = isinstance(other, Tuple)
         if isTuple and len(other) != 2:
             raise ValueError('expecting Tuple of length 2')
 
         # If the other is not an Expression, and neither a Tuple
-        if not isinstance(other, Expr) and not isinstance(other, Tuple):
+        if not isinstance(other, (Expr, Tuple)):
             return S.false
         # self in rectangular form
         if not self.polar:
@@ -1282,17 +1367,15 @@ class ComplexRegion(Set):
 
 
 class CartesianComplexRegion(ComplexRegion):
-    """
+    r"""
     Set representing a square region of the complex plane.
 
-    Z = {z in C | z = x + I*y, x in [Re(z)], y in [Im(z)]}
+    .. math:: Z = \{z \in \mathbb{C} \mid z = x + Iy, x \in [\operatorname{re}(z)], y \in [\operatorname{im}(z)]\}
 
     Examples
     ========
 
-    >>> from sympy.sets.fancysets import ComplexRegion
-    >>> from sympy.sets.sets import Interval
-    >>> from sympy import I
+    >>> from sympy import ComplexRegion, I, Interval
     >>> region = ComplexRegion(Interval(1, 3) * Interval(4, 6))
     >>> 2 + 5*I in region
     True
@@ -1338,16 +1421,15 @@ class CartesianComplexRegion(ComplexRegion):
 
 
 class PolarComplexRegion(ComplexRegion):
-    """
+    r"""
     Set representing a polar region of the complex plane.
 
-    Z = {z in C | z = r*[cos(theta) + I*sin(theta)], r in [r], theta in [theta]}
+    .. math:: Z = \{z \in \mathbb{C} \mid z = r\times (\cos(\theta) + I\sin(\theta)), r \in [\texttt{r}], \theta \in [\texttt{theta}]\}
 
     Examples
     ========
 
-    >>> from sympy.sets.fancysets import ComplexRegion, Interval
-    >>> from sympy import oo, pi, I
+    >>> from sympy import ComplexRegion, Interval, oo, pi, I
     >>> rset = Interval(0, oo)
     >>> thetaset = Interval(0, pi)
     >>> upper_half_plane = ComplexRegion(rset * thetaset, polar=True)
@@ -1394,7 +1476,7 @@ class PolarComplexRegion(ComplexRegion):
 
 class Complexes(CartesianComplexRegion, metaclass=Singleton):
     """
-    The Set of all complex numbers
+    The :class:`Set` of all complex numbers
 
     Examples
     ========
@@ -1423,9 +1505,3 @@ class Complexes(CartesianComplexRegion, metaclass=Singleton):
 
     def __new__(cls):
         return Set.__new__(cls)
-
-    def __str__(self):
-        return "S.Complexes"
-
-    def __repr__(self):
-        return "S.Complexes"
