@@ -105,15 +105,16 @@ See the appropriate docstrings for a detailed explanation of the output.
 #      - Idx with step determined by function call
 from collections.abc import Iterable
 
-from sympy import Number
+from sympy.core.numbers import Number
 from sympy.core.assumptions import StdFactKB
 from sympy.core import Expr, Tuple, sympify, S
 from sympy.core.symbol import _filter_assumptions, Symbol
-from sympy.core.compatibility import (is_sequence, NotIterable)
 from sympy.core.logic import fuzzy_bool, fuzzy_not
 from sympy.core.sympify import _sympify
 from sympy.functions.special.tensor_functions import KroneckerDelta
 from sympy.multipledispatch import dispatch
+from sympy.utilities.iterables import is_sequence, NotIterable
+from sympy.utilities.misc import filldedent
 
 
 class IndexException(Exception):
@@ -144,7 +145,6 @@ class Indexed(Expr):
     is_Atom = True
 
     def __new__(cls, base, *args, **kw_args):
-        from sympy.utilities.misc import filldedent
         from sympy.tensor.array.ndim_array import NDimArray
         from sympy.matrices.matrices import MatrixBase
 
@@ -159,12 +159,14 @@ class Indexed(Expr):
                 items (i.e. an object with a `__getitem__` method).
                 """))
         args = list(map(sympify, args))
-        if isinstance(base, (NDimArray, Iterable, Tuple, MatrixBase)) and all([i.is_number for i in args]):
+        if isinstance(base, (NDimArray, Iterable, Tuple, MatrixBase)) and all(i.is_number for i in args):
             if len(args) == 1:
                 return base[args[0]]
             else:
                 return base[args]
 
+        base = _sympify(base)
+
         obj = Expr.__new__(cls, base, *args, **kw_args)
 
         try:
@@ -283,7 +285,6 @@ class Indexed(Expr):
         >>> B[i, j].shape
         (m, m)
         """
-        from sympy.utilities.misc import filldedent
 
         if self.base.shape:
             return self.base.shape
@@ -324,12 +325,12 @@ class Indexed(Expr):
 
         """
         ranges = []
+        sentinel = object()
         for i in self.indices:
-            sentinel = object()
             upper = getattr(i, 'upper', sentinel)
             lower = getattr(i, 'lower', sentinel)
             if sentinel not in (upper, lower):
-                ranges.append(Tuple(lower, upper))
+                ranges.append((lower, upper))
             else:
                 ranges.append(None)
         return ranges
@@ -350,6 +351,14 @@ class Indexed(Expr):
 
     @property
     def expr_free_symbols(self):
+        from sympy.utilities.exceptions import sympy_deprecation_warning
+        sympy_deprecation_warning("""
+        The expr_free_symbols property is deprecated. Use free_symbols to get
+        the free symbols of an expression.
+        """,
+            deprecated_since_version="1.9",
+            active_deprecations_target="deprecated-expr-free-symbols")
+
         return {self}
 
 
@@ -433,7 +442,8 @@ class IndexedBase(Expr, NotIterable):
         obj._assumptions._generator = tmp_asm_copy  # Issue #8873
 
     def __new__(cls, label, shape=None, *, offset=S.Zero, strides=None, **kw_args):
-        from sympy import MatrixBase, NDimArray
+        from sympy.matrices.matrices import MatrixBase
+        from sympy.tensor.array.ndim_array import NDimArray
 
         assumptions, kw_args = _filter_assumptions(kw_args)
         if isinstance(label, str):
@@ -636,7 +646,6 @@ class Idx(Expr):
     _diff_wrt = True
 
     def __new__(cls, label, range=None, **kw_args):
-        from sympy.utilities.misc import filldedent
 
         if isinstance(label, str):
             label = Symbol(label, integer=True)
